Class: Faye::Redis::SubscriptionManager
- Inherits:
-
Object
- Object
- Faye::Redis::SubscriptionManager
- Defined in:
- lib/faye/redis/subscription_manager.rb
Instance Attribute Summary collapse
-
#connection ⇒ Object
readonly
Returns the value of attribute connection.
-
#options ⇒ Object
readonly
Returns the value of attribute options.
Instance Method Summary collapse
-
#channel_matches_pattern?(channel, pattern) ⇒ Boolean
Check if a channel matches a pattern.
-
#cleanup_client_subscriptions(client_id) ⇒ Object
Clean up subscriptions for a client.
-
#cleanup_orphaned_data(active_client_ids, &callback) ⇒ Object
Comprehensive cleanup of orphaned subscription data This should be called periodically during garbage collection Processes in batches to avoid blocking the connection pool.
-
#get_client_subscriptions(client_id, &callback) ⇒ Object
Get all channels a client is subscribed to.
-
#get_pattern_subscribers(channel) ⇒ Object
Get subscribers matching wildcard patterns.
-
#get_subscribers(channel, &callback) ⇒ Object
Get all clients subscribed to a channel.
-
#initialize(connection, options = {}) ⇒ SubscriptionManager
constructor
A new instance of SubscriptionManager.
-
#subscribe(client_id, channel, &callback) ⇒ Object
Subscribe a client to a channel.
-
#unsubscribe(client_id, channel, &callback) ⇒ Object
Unsubscribe a client from a channel.
-
#unsubscribe_all(client_id, &callback) ⇒ Object
Unsubscribe a client from all channels.
Constructor Details
#initialize(connection, options = {}) ⇒ SubscriptionManager
Returns a new instance of SubscriptionManager.
6 7 8 9 |
# File 'lib/faye/redis/subscription_manager.rb', line 6 def initialize(connection, = {}) @connection = connection @options = end |
Instance Attribute Details
#connection ⇒ Object (readonly)
Returns the value of attribute connection.
4 5 6 |
# File 'lib/faye/redis/subscription_manager.rb', line 4 def connection @connection end |
#options ⇒ Object (readonly)
Returns the value of attribute options.
4 5 6 |
# File 'lib/faye/redis/subscription_manager.rb', line 4 def @options end |
Instance Method Details
#channel_matches_pattern?(channel, pattern) ⇒ Boolean
Check if a channel matches a pattern
162 163 164 165 166 167 168 169 170 171 172 |
# File 'lib/faye/redis/subscription_manager.rb', line 162 def channel_matches_pattern?(channel, pattern) # Convert Faye wildcard pattern to regex # * matches one segment, ** matches multiple segments regex_pattern = pattern .gsub('**', '__DOUBLE_STAR__') .gsub('*', '[^/]+') .gsub('__DOUBLE_STAR__', '.*') regex = Regexp.new("^#{regex_pattern}$") !!(channel =~ regex) end |
#cleanup_client_subscriptions(client_id) ⇒ Object
Clean up subscriptions for a client
175 176 177 |
# File 'lib/faye/redis/subscription_manager.rb', line 175 def cleanup_client_subscriptions(client_id) unsubscribe_all(client_id) end |
#cleanup_orphaned_data(active_client_ids, &callback) ⇒ Object
Comprehensive cleanup of orphaned subscription data This should be called periodically during garbage collection Processes in batches to avoid blocking the connection pool
182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 |
# File 'lib/faye/redis/subscription_manager.rb', line 182 def cleanup_orphaned_data(active_client_ids, &callback) active_set = active_client_ids.to_set namespace = @options[:namespace] || 'faye' batch_size = @options[:cleanup_batch_size] || 50 # Phase 1: Scan for orphaned subscriptions scan_orphaned_subscriptions(active_set, namespace) do |orphaned_subscriptions| # Phase 2: Clean up orphaned subscriptions in batches cleanup_orphaned_subscriptions_batched(orphaned_subscriptions, namespace, batch_size) do # Phase 3: Clean up empty channels (yields between operations) cleanup_empty_channels_async(namespace) do # Phase 4: Clean up unused patterns cleanup_unused_patterns_async do callback.call if callback end end end end rescue => e log_error("Failed to cleanup orphaned data: #{e.}") EventMachine.next_tick { callback.call } if callback end |
#get_client_subscriptions(client_id, &callback) ⇒ Object
Get all channels a client is subscribed to
102 103 104 105 106 107 108 109 110 111 112 113 |
# File 'lib/faye/redis/subscription_manager.rb', line 102 def get_client_subscriptions(client_id, &callback) channels = @connection.with_redis do |redis| redis.smembers(client_subscriptions_key(client_id)) end EventMachine.next_tick { callback.call(channels) } if callback channels rescue => e log_error("Failed to get subscriptions for client #{client_id}: #{e.}") EventMachine.next_tick { callback.call([]) } if callback [] end |
#get_pattern_subscribers(channel) ⇒ Object
Get subscribers matching wildcard patterns
136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 |
# File 'lib/faye/redis/subscription_manager.rb', line 136 def get_pattern_subscribers(channel) patterns = @connection.with_redis do |redis| redis.smembers(patterns_key) end # Filter to only matching patterns first matching_patterns = patterns.select { |pattern| channel_matches_pattern?(channel, pattern) } return [] if matching_patterns.empty? # Use pipelining to fetch all matching pattern subscribers in one network round-trip results = @connection.with_redis do |redis| redis.pipelined do |pipeline| matching_patterns.each do |pattern| pipeline.smembers(channel_subscribers_key(pattern)) end end end # Flatten and deduplicate results results.flatten.uniq rescue => e log_error("Failed to get pattern subscribers for channel #{channel}: #{e.}") [] end |
#get_subscribers(channel, &callback) ⇒ Object
Get all clients subscribed to a channel
116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 |
# File 'lib/faye/redis/subscription_manager.rb', line 116 def get_subscribers(channel, &callback) # Get direct subscribers direct_subscribers = @connection.with_redis do |redis| redis.smembers(channel_subscribers_key(channel)) end # Get pattern subscribers pattern_subscribers = get_pattern_subscribers(channel) all_subscribers = (direct_subscribers + pattern_subscribers).uniq EventMachine.next_tick { callback.call(all_subscribers) } if callback all_subscribers rescue => e log_error("Failed to get subscribers for channel #{channel}: #{e.}") EventMachine.next_tick { callback.call([]) } if callback [] end |
#subscribe(client_id, channel, &callback) ⇒ Object
Subscribe a client to a channel
12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 |
# File 'lib/faye/redis/subscription_manager.rb', line 12 def subscribe(client_id, channel, &callback) = Time.now.to_i subscription_ttl = @options[:subscription_ttl] || 86400 # 24 hours default @connection.with_redis do |redis| redis.multi do |multi| # Add channel to client's subscriptions multi.sadd?(client_subscriptions_key(client_id), channel) # Set/refresh TTL for client subscriptions list multi.expire(client_subscriptions_key(client_id), subscription_ttl) # Add client to channel's subscribers multi.sadd?(channel_subscribers_key(channel), client_id) # Set/refresh TTL for channel subscribers list multi.expire(channel_subscribers_key(channel), subscription_ttl) # Store subscription metadata multi.hset( subscription_key(client_id, channel), 'subscribed_at', , 'channel', channel, 'client_id', client_id ) # Set TTL for subscription metadata multi.expire(subscription_key(client_id, channel), subscription_ttl) # Handle wildcard patterns if channel.include?('*') multi.sadd?(patterns_key, channel) # Set/refresh TTL for patterns set multi.expire(patterns_key, subscription_ttl) end end end EventMachine.next_tick { callback.call(true) } if callback rescue => e log_error("Failed to subscribe client #{client_id} to #{channel}: #{e.}") EventMachine.next_tick { callback.call(false) } if callback end |
#unsubscribe(client_id, channel, &callback) ⇒ Object
Unsubscribe a client from a channel
54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 |
# File 'lib/faye/redis/subscription_manager.rb', line 54 def unsubscribe(client_id, channel, &callback) @connection.with_redis do |redis| redis.multi do |multi| # Remove channel from client's subscriptions multi.srem?(client_subscriptions_key(client_id), channel) # Remove client from channel's subscribers multi.srem?(channel_subscribers_key(channel), client_id) # Delete subscription metadata multi.del(subscription_key(client_id, channel)) end end # Clean up wildcard pattern if no more subscribers if channel.include?('*') cleanup_pattern_if_unused(channel) end EventMachine.next_tick { callback.call(true) } if callback rescue => e log_error("Failed to unsubscribe client #{client_id} from #{channel}: #{e.}") EventMachine.next_tick { callback.call(false) } if callback end |
#unsubscribe_all(client_id, &callback) ⇒ Object
Unsubscribe a client from all channels
80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 |
# File 'lib/faye/redis/subscription_manager.rb', line 80 def unsubscribe_all(client_id, &callback) # Get all channels the client is subscribed to get_client_subscriptions(client_id) do |channels| if channels.empty? callback.call(true) if callback else # Unsubscribe from each channel remaining = channels.size channels.each do |channel| unsubscribe(client_id, channel) do remaining -= 1 callback.call(true) if callback && remaining == 0 end end end end rescue => e log_error("Failed to unsubscribe client #{client_id} from all channels: #{e.}") EventMachine.next_tick { callback.call(false) } if callback end |